F(total_sessions)
F(foreign_sessions)
F(transcoded_media)
F(ipv4_sessions)
F(ipv6_sessions)
F(mixed_sessions)
F(userspace_streams)
F(kernel_only_streams)
F(kernel_user_streams)
F(media_cache)
F(player_cache)
